@@ -3,10 +3,9 @@
 # as part of this package.
 #
 
-"""
-This is an implementation of a state-emitting MarkovModel.  I am using
-terminology similar to Manning and Schutze.
+"""A state-emitting MarkovModel.
 
+Note terminology similar to Manning and Schutze is used.
 
 
 Functions:
@@ -23,6 +22,7 @@ MarkovModel     Holds the description of a markov model
 
 import numpy
 
+
 try:
     logaddexp = numpy.logaddexp
 except AttributeError:
@@ -34,6 +34,7 @@ except AttributeError:
     warnings.warn("For optimal speed, please update to Numpy version 1.3 or later (current version is %s)" % numpy.__version__)
 
     def logaddexp(logx, logy):
+        """Implement logaddexp method if Numpy version is older than 1.3."""
         if logy - logx > 100:
             return logy
         elif logx - logy > 100:
@@ -43,6 +44,7 @@ except AttributeError:
 
 
 def itemindex(values):
+    """Return a dictionary of values with their sequence offset as keys."""
     d = {}
     entries = enumerate(values[::-1])
     n = len(values) - 1
@@ -50,6 +52,7 @@ def itemindex(values):
         d[key] = n - index
     return d
 
+
 numpy.random.seed()
 
 VERY_SMALL_NUMBER = 1E-300
@@ -57,8 +60,11 @@ LOG0 = numpy.log(VERY_SMALL_NUMBER)
 
 
 class MarkovModel(object):
+    """Create a state-emitting MarkovModel object."""
+
     def __init__(self, states, alphabet,
                  p_initial=None, p_transition=None, p_emission=None):
+        """Initialize the class."""
         self.states = states
         self.alphabet = alphabet
         self.p_initial = p_initial
@@ -66,6 +72,7 @@ class MarkovModel(object):
         self.p_emission = p_emission
 
     def __str__(self):
+        """Create a string representation of the MarkovModel object."""
         from Bio._py3k import StringIO
         handle = StringIO()
         save(self, handle)
@@ -74,6 +81,7 @@ class MarkovModel(object):
 
 
 def _readline_and_check_start(handle, start):
+    """Read the first line and evaluate that begisn with the correct start (PRIVATE)."""
     line = handle.readline()
     if not line.startswith(start):
         raise ValueError("I expected %r but got %r" % (start, line))
@@ -81,7 +89,7 @@ def _readline_and_check_start(handle, start):
 
 
 def load(handle):
-    """load(handle) -> MarkovModel()"""
+    """Parse a file handle into a MarkovModel object."""
     # Load the states.
     line = _readline_and_check_start(handle, "STATES:")
     states = line.split()[1:]
@@ -118,7 +126,7 @@ def load(handle):
 
 
 def save(mm, handle):
-    """save(mm, handle)"""
+    """Save MarkovModel object into handle."""
     # This will fail if there are spaces in the states or alphabet.
     w = handle.write
     w("STATES: %s\n" % ' '.join(mm.states))
@@ -139,8 +147,7 @@ def train_bw(states, alphabet, training_data,
              pseudo_initial=None, pseudo_transition=None, pseudo_emission=None,
              update_fn=None,
              ):
-    """train_bw(states, alphabet, training_data[, pseudo_initial]
-    [, pseudo_transition][, pseudo_emission][, update_fn]) -> MarkovModel
+    """Train a MarkovModel using the Baum-Welch algorithm.
 
     Train a MarkovModel using the Baum-Welch algorithm.  states is a list
     of strings that describe the names of each state.  alphabet is a
@@ -156,7 +163,6 @@ def train_bw(states, alphabet, training_data,
 
     update_fn is an optional callback that takes parameters
     (iteration, log_likelihood).  It is called once per iteration.
-
     """
     N, M = len(states), len(alphabet)
     if not training_data:
@@ -198,6 +204,7 @@ def train_bw(states, alphabet, training_data,
     p_initial, p_transition, p_emission = x
     return MarkovModel(states, alphabet, p_initial, p_transition, p_emission)
 
+
 MAX_ITERATIONS = 1000
 
 
@@ -205,7 +212,7 @@ def _baum_welch(N, M, training_outputs,
                 p_initial=None, p_transition=None, p_emission=None,
                 pseudo_initial=None, pseudo_transition=None,
                 pseudo_emission=None, update_fn=None):
-    # Returns (p_initial, p_transition, p_emission)
+    """Implement the Baum-Welch algorithm to evaluate unknown parameters in the MarkovModel object (PRIVATE)."""
     if p_initial is None:
         p_initial = _random_norm(N)
     else:
@@ -244,11 +251,10 @@ def _baum_welch(N, M, training_outputs,
     for i in range(MAX_ITERATIONS):
         llik = LOG0
         for outputs in training_outputs:
-            x = _baum_welch_one(
+            llik += _baum_welch_one(
                 N, M, outputs,
                 lp_initial, lp_transition, lp_emission,
                 lpseudo_initial, lpseudo_transition, lpseudo_emission,)
-            llik += x
         if update_fn is not None:
             update_fn(i, llik)
         if prev_llik is not None and numpy.fabs(prev_llik - llik) < 0.1:
@@ -259,15 +265,17 @@ def _baum_welch(N, M, training_outputs,
                            % MAX_ITERATIONS)
 
     # Return everything back in normal space.
-    return [numpy.exp(x) for x in (lp_initial, lp_transition, lp_emission)]
+    return [numpy.exp(_) for _ in (lp_initial, lp_transition, lp_emission)]
 
 
 def _baum_welch_one(N, M, outputs,
                     lp_initial, lp_transition, lp_emission,
                     lpseudo_initial, lpseudo_transition, lpseudo_emission):
-    # Do one iteration of Baum-Welch based on a sequence of output.
-    # NOTE: This will change the values of lp_initial, lp_transition,
-    # and lp_emission in place.
+    """Execute one step for Baum-Welch algorithm (PRIVATE).
+
+    Do one iteration of Baum-Welch based on a sequence of output.
+    Changes the value for lp_initial, lp_transition and lp_emission in place.
+    """
     T = len(outputs)
     fmat = _forward(N, T, lp_initial, lp_transition, lp_emission, outputs)
     bmat = _backward(N, T, lp_transition, lp_emission, outputs)
@@ -284,10 +292,10 @@ def _baum_welch_one(N, M, outputs,
                 # P(making this transition)
                 # P(emitting this character)
                 # P(going to the end)
-                lp = fmat[i][t] + \
-                     lp_transition[i][j] + \
-                     lp_emission[i][k] + \
-                     bmat[j][t + 1]
+                lp = (fmat[i][t] +
+                      lp_transition[i][j] +
+                      lp_emission[i][k] +
+                      bmat[j][t + 1])
                 lp_traverse[i][j] = lp
         # Normalize the probability for this time step.
         lp_arc[:, :, t] = lp_traverse - _logsum(lp_traverse)
@@ -345,10 +353,11 @@ def _baum_welch_one(N, M, outputs,
 
 
 def _forward(N, T, lp_initial, lp_transition, lp_emission, outputs):
-    # Implement the forward algorithm.  This actually calculates a
-    # Nx(T+1) matrix, where the last column is the total probability
-    # of the output.
+    """Implement forward algorithm (PRIVATE).
 
+    Calculate a Nx(T+1) matrix, where the last column is the total
+    probability of the output.
+    """
     matrix = numpy.zeros((N, T + 1))
 
     # Initialize the first column to be the initial values.
@@ -360,15 +369,14 @@ def _forward(N, T, lp_initial, lp_transition, lp_emission, outputs):
             # transitions from all the states from time t-1.
             lprob = LOG0
             for i in range(N):
-                lp = matrix[i][t - 1] + \
-                     lp_transition[i][j] + \
-                     lp_emission[i][k]
+                lp = matrix[i][t - 1] + lp_transition[i][j] + lp_emission[i][k]
                 lprob = logaddexp(lprob, lp)
             matrix[j][t] = lprob
     return matrix
 
 
 def _backward(N, T, lp_transition, lp_emission, outputs):
+    """Implement backward algorithm (PRIVATE)."""
     matrix = numpy.zeros((N, T + 1))
     for t in range(T - 1, -1, -1):
         k = outputs[t]
@@ -377,9 +385,9 @@ def _backward(N, T, lp_transition, lp_emission, outputs):
             # transitions from all the states from time t+1.
             lprob = LOG0
             for j in range(N):
-                lp = matrix[j][t + 1] + \
-                     lp_transition[i][j] + \
-                     lp_emission[i][k]
+                lp = (matrix[j][t + 1] +
+                      lp_transition[i][j] +
+                      lp_emission[i][k])
                 lprob = logaddexp(lprob, lp)
             matrix[i][t] = lprob
     return matrix
@@ -388,8 +396,7 @@ def _backward(N, T, lp_transition, lp_emission, outputs):
 def train_visible(states, alphabet, training_data,
                   pseudo_initial=None, pseudo_transition=None,
                   pseudo_emission=None):
-    """train_visible(states, alphabet, training_data[, pseudo_initial]
-    [, pseudo_transition][, pseudo_emission]) -> MarkovModel
+    """Train a visible MarkovModel using maximum likelihoood estimates for each of the parameters.
 
     Train a visible MarkovModel using maximum likelihoood estimates
     for each of the parameters.  states is a list of strings that
@@ -402,8 +409,7 @@ def train_visible(states, alphabet, training_data,
     pseudo_initial, pseudo_transition, and pseudo_emission are
     optional parameters that you can use to assign pseudo-counts to
     different matrices.  They should be matrices of the appropriate
-    size that contain numbers to add to each parameter matrix
-
+    size that contain numbers to add to each parameter matrix.
     """
     N, M = len(states), len(alphabet)
     if pseudo_initial is not None:
@@ -442,6 +448,7 @@ def train_visible(states, alphabet, training_data,
 
 def _mle(N, M, training_outputs, training_states, pseudo_initial,
          pseudo_transition, pseudo_emission):
+    """Implement Maximum likelihood estimation algorithm (PRIVATE)."""
     # p_initial is the probability that a sequence of states starts
     # off with a particular one.
     p_initial = numpy.zeros(N)
@@ -479,11 +486,15 @@ def _mle(N, M, training_outputs, training_states, pseudo_initial,
 
 
 def _argmaxes(vector, allowance=None):
+    """Return indeces of the maximum values aong the vector (PRIVATE)."""
     return [numpy.argmax(vector)]
 
 
 def find_states(markov_model, output):
-    """find_states(markov_model, output) -> list of (states, score)"""
+    """Find states in the given Markov model output.
+
+    Returns a list of (states, score) tuples.
+    """
     mm = markov_model
     N = len(mm.states)
 
@@ -506,9 +517,7 @@ def find_states(markov_model, output):
 
 
 def _viterbi(N, lp_initial, lp_transition, lp_emission, output):
-    # The Viterbi algorithm finds the most likely set of states for a
-    # given output.  Returns a list of states.
-
+    """Implement Viterbi algorithm to find most likely states for a given input (PRIVATE)."""
     T = len(output)
     # Store the backtrace in a NxT matrix.
     backtrace = []    # list of indexes of states in previous timestep.
@@ -522,9 +531,9 @@ def _viterbi(N, lp_initial, lp_transition, lp_emission, output):
         k = output[t]
         for j in range(N):
             # Find the most likely place it came from.
-            i_scores = scores[:, t - 1] + \
-                       lp_transition[:, j] + \
-                       lp_emission[j, k]
+            i_scores = (scores[:, t - 1] +
+                        lp_transition[:, j] +
+                        lp_emission[j, k])
             indexes = _argmaxes(i_scores)
             scores[j, t] = i_scores[indexes[0]]
             backtrace[j][t] = indexes
@@ -551,7 +560,7 @@ def _viterbi(N, lp_initial, lp_transition, lp_emission, output):
 
 
 def _normalize(matrix):
-    # Make sure numbers add up to 1.0
+    """Normalize matrix object (PRIVATE)."""
     if len(matrix.shape) == 1:
         matrix = matrix / float(sum(matrix))
     elif len(matrix.shape) == 2:
@@ -564,16 +573,19 @@ def _normalize(matrix):
 
 
 def _uniform_norm(shape):
+    """Normalize a uniform matrix (PRIVATE)."""
     matrix = numpy.ones(shape)
     return _normalize(matrix)
 
 
 def _random_norm(shape):
+    """Normalize a random matrix (PRIVATE)."""
     matrix = numpy.random.random(shape)
     return _normalize(matrix)
 
 
 def _copy_and_check(matrix, desired_shape):
+    """Copy a matrix and check its dimension. Normalize at the end (PRIVATE)."""
     # Copy the matrix.
     matrix = numpy.array(matrix, copy=1)
     # Check the dimensions.
@@ -593,6 +605,7 @@ def _copy_and_check(matrix, desired_shape):
 
 
 def _logsum(matrix):
+    """Implement logsum for a matrix object (PRIVATE)."""
     if len(matrix.shape) > 1:
         vec = numpy.reshape(matrix, (numpy.product(matrix.shape),))
     else:
@@ -604,6 +617,7 @@ def _logsum(matrix):
 
 
 def _logvecadd(logvec1, logvec2):
+    """Implement a log sum for two vector objects (PRIVATE)."""
     assert len(logvec1) == len(logvec2), "vectors aren't the same length"
     sumvec = numpy.zeros(len(logvec1))
     for i in range(len(logvec1)):
@@ -612,5 +626,6 @@ def _logvecadd(logvec1, logvec2):
 
 
 def _exp_logsum(numbers):
+    """Return the exponential of a logsum (PRIVATE)."""
     sum = _logsum(numbers)
     return numpy.exp(sum)
